
#include <fstream>
#include <cmath>

template <typename T>
inline T sqr(T t) { return t*t; }

const long double eps = 1E-10;
const int precision = 15;

struct Vector
{
  long double x, y, z;
  Vector () {}
  Vector (long double xi, long double yi, long double zi): x(xi), y(yi), z(zi) {}
  Vector operator* (const Vector& rhs) const
  {
    return Vector(this->y*rhs.z - this->z*rhs.y, -this->x*rhs.z + this->z*rhs.x, this->x*rhs.y - this->y*rhs.x);
  }
  long double operator& (const Vector& rhs) const
  {
    return this->x*rhs.x + this->y*rhs.y + this->z*rhs.z;
  }
  Vector operator* (long double c) const
  {
    return Vector(this->x*c, this->y*c, this->z*c);
  }
  Vector operator/ (long double c) const
  {
    return *this * (1 / c);
  }
  long double abs() const
  {
    return sqrt(x*x + y*y + z*z);
  }
  Vector operator+ (const Vector& rhs) const
  {
    return Vector(this->x + rhs.x, this->y + rhs.y, this->z + rhs.z);
  }
  Vector operator- (const Vector& rhs) const
  {
    return *this + (rhs * -1);
  }
  Vector norm() const
  {
    if (abs() > eps) 
      return Vector(*this) / abs();
    else
      return Vector(*this);
  }
};

std::istream& operator>> (std::istream& stream, Vector& v)
{
  return stream >> v.x >> v.y >> v.z;
}
std::ostream& operator<< (std::ostream& stream, const Vector& v)
{
  return stream << v.x << " " << v.y << " " << v.z;
}

int main(int argc, char** argv)
{
  Vector torus_center, torus_direction, ship;
  long double r1, r2;
  std::fstream input("input.txt", std::ios_base::in);
  std::fstream output("output.txt", std::ios_base::out);
  long double h, r, route_len, cos_alpha;
  Vector u, v, w, res;

  input >> torus_center >> torus_direction >> r1 >> r2 >> ship;

  torus_direction = torus_direction.norm();
  ship = ship - torus_center;
  
  h = ship & torus_direction;
  r = (ship * torus_direction).abs();

  route_len = sqrt(sqr(h) + sqr(r - r1)) - r2;

  cos_alpha = (ship.abs() > eps) ? (sqr(ship.abs()) + sqr(route_len + r2) - sqr(r1))/(2*ship.abs()*(route_len + r2)) : 0;
  if (cos_alpha < -1) cos_alpha = -1;
  if (cos_alpha > 1) cos_alpha = 1;
  w = ship - ship.norm() * cos_alpha * route_len;

  if (r < eps)
    u = torus_direction * ((std::abs(torus_direction.x) < eps) ? Vector(1, 0, 0) : Vector(0, 1, 1));
  else if (std::abs(h) < eps)
    u = ship;
  else 
    u = (ship - torus_direction * h) * ship * ship;

  res = u.norm() * -route_len * sqrt(1 - sqr(cos_alpha)) + w + torus_center;

  output.precision(precision);
  output << res;

  return 0;
}
